"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _execa = _interopRequireDefault(require("execa"));

var _dotProp = _interopRequireDefault(require("dot-prop"));

var _ink = require("ink");

var _inkTextInput = _interopRequireDefault(require("ink-text-input"));

var _inkSelectInput = _interopRequireDefault(require("ink-select-input"));

var _inkSpinner = _interopRequireDefault(require("ink-spinner"));

var _utils = require("./utils");

var _algoliaSearch = require("./libs/algoliaSearch");

var _npmSuggestions = require("./libs/npm-suggestions");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

// Progress
const PROGRESS_NOT_LOADED = 0;
const PROGRESS_LOADING = 1;
const PROGRESS_LOADED = 2;
const PROGRESS_ERROR = 3; // Focused

const FOCUSED_SEARCH = 0;
const FOCUSED_SUGGESTIONS = 1; // Info

const EmptyQuery = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, 'Try typing in to search the database.'));

const NoSelectedPackages = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, 'Select packages to get suggestions.'));

const NotFoundSearchInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, "We couldn't find any package that would match your input..."));

const NotFoundSuggestionsInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  grey: true
}, "We couldn't suggest any package that would match your picked packages..."));

const ErrorInfo = ({
  err
}) => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  red: true
}, 'Error: ', err));

const AlgoliaInfo = () => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, null, 'Search powered by'), (0, _ink.h)(_ink.Text, {
  blue: true
}, ' Algolia'), (0, _ink.h)(_ink.Text, null, '.')); // Package


const PackageAttribute = (_ref) => {
  let {
    pkg,
    attr
  } = _ref,
      props = _objectWithoutProperties(_ref, ["pkg", "attr"]);

  return (0, _ink.h)(_ink.Text, props, `${_dotProp.default.get(pkg, attr)} ${pkg._cell(attr)}`.slice(0, (0, _utils.maxCellSize)()));
};

const Package = pkg => (0, _ink.h)(_ink.Text, null, (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "humanDownloadsLast30Days"
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "name",
  blueBright: true,
  bold: true
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "owner.name",
  cyan: true
}), (0, _ink.h)(PackageAttribute, {
  pkg: pkg,
  attr: "description",
  bold: true
})); // Search


const Search = ({
  value,
  onChange
}) => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  bold: true
}, `Search packages 📦  : `), (0, _ink.h)(_inkTextInput.default, {
  value: value,
  onChange: onChange,
  placeholder: "..."
})); // Overview


const SelectedPackage = ({
  pkg
}) => (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  magenta: true
}, ` ›`), (0, _ink.h)(_ink.Text, {
  bold: true
}, ` ${pkg.name} `), (0, _ink.h)(_ink.Text, {
  grey: true
}, ` ${pkg.version} `));

const SelectedPackages = ({
  selectedPackages
}) => (0, _ink.h)("div", null, (0, _ink.h)("div", null), (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
  bold: true
}, 'Picked:'), (0, _ink.h)(_ink.Text, {
  grey: true
}, ' Press Space to install packages...')), selectedPackages.map(pkg => (0, _ink.h)(SelectedPackage, {
  key: pkg.name,
  pkg: pkg
}))); // Restults


const SearchResults = ({
  foundPackages,
  onToggle,
  loading,
  focused
}) => {
  if (loading === PROGRESS_LOADING) {
    return (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
      bold: true
    }, (0, _ink.h)(_inkSpinner.default, {
      red: true
    }), ' Fetching search'));
  }

  if (loading === PROGRESS_ERROR) {
    return (0, _ink.h)(ErrorInfo, {
      err: "Couldn't reach Algolia search!"
    });
  }

  if ((0, _utils.isEmpty)(foundPackages)) {
    return (0, _ink.h)(NotFoundSearchInfo, null);
  }

  return (0, _ink.h)("span", null, (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
    bold: true
  }, 'Search results:'), !focused && (0, _ink.h)(_ink.Text, {
    grey: true
  }, ' Press Tab to select search results')), focused ? (0, _ink.h)(_inkSelectInput.default, {
    items: foundPackages,
    itemComponent: Package,
    onSelect: onToggle
  }) : foundPackages.map(pkg => (0, _ink.h)("div", null, " ", Package(pkg))));
};

const SuggestionsResults = ({
  foundPackages,
  onToggle,
  loading,
  focused
}) => {
  if (loading === PROGRESS_LOADING) {
    return (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
      bold: true
    }, (0, _ink.h)(_inkSpinner.default, {
      red: true
    }), ' Fetching suggestions'));
  }

  if (loading === PROGRESS_ERROR) {
    return (0, _ink.h)(ErrorInfo, {
      err: "Couldn't get suggestions!"
    });
  }

  if ((0, _utils.isEmpty)(foundPackages)) {
    return (0, _ink.h)(NotFoundSuggestionsInfo, null);
  }

  return (0, _ink.h)("span", null, (0, _ink.h)("div", null, (0, _ink.h)(_ink.Text, {
    bold: true
  }, 'Suggestions results:'), !focused && (0, _ink.h)(_ink.Text, {
    grey: true
  }, ' Press Tab to select suggestions')), focused ? (0, _ink.h)(_inkSelectInput.default, {
    items: foundPackages,
    itemComponent: Package,
    onSelect: onToggle
  }) : foundPackages.map(pkg => (0, _ink.h)("div", null, " ", Package(pkg))));
}; // Emma


class Emma extends _ink.Component {
  constructor(props) {
    super(props);
    this.state = {
      query: '',
      foundSearchPackages: [],
      foundSuggestionsPackages: [],
      selectedPackages: [],
      loadingSearch: PROGRESS_NOT_LOADED,
      loadingSearchBackground: false,
      loadingSuggestions: PROGRESS_NOT_LOADED,
      focused: FOCUSED_SEARCH
    };
    this.handleKeyPress = this.handleKeyPress.bind(this);
    this.handleQueryChange = this.handleQueryChange.bind(this);
    this.handleTogglePackage = this.handleTogglePackage.bind(this);
    this.fetchSearch = this.fetchSearch.bind(this);
    this.fetchSuggestions = this.fetchSuggestions.bind(this);
    this.handleInstall = this.handleInstall.bind(this);
  }

  componentDidMount() {
    process.stdin.on('keypress', this.handleKeyPress);
  }

  componentWillUnmount() {
    process.stdin.removeListener('keypress', this.handleKeyPress);
  }

  render() {
    const {
      query,
      foundSearchPackages,
      foundSuggestionsPackages,
      selectedPackages,
      loadingSearch,
      loadingSuggestions,
      focused
    } = this.state;
    return (0, _ink.h)("div", null, (0, _ink.h)(Search, {
      value: query,
      onChange: this.handleQueryChange,
      loading: loadingSearch
    }), (0, _utils.isEmpty)(query) ? (0, _ink.h)(EmptyQuery, null) : (0, _ink.h)(SearchResults, {
      foundPackages: foundSearchPackages,
      onToggle: this.handleTogglePackage,
      loading: loadingSearch,
      focused: focused === FOCUSED_SEARCH
    }), (0, _utils.isEmpty)(selectedPackages) ? (0, _ink.h)(NoSelectedPackages, null) : (0, _ink.h)(SuggestionsResults, {
      foundPackages: foundSuggestionsPackages,
      onToggle: this.handleTogglePackage,
      loading: loadingSuggestions,
      focused: focused === FOCUSED_SUGGESTIONS
    }), (0, _ink.h)(AlgoliaInfo, null), (0, _utils.notEmpty)(selectedPackages) && (0, _ink.h)(SelectedPackages, {
      selectedPackages: selectedPackages
    }));
  }

  handleKeyPress(_, key) {
    var _this = this;

    return _asyncToGenerator(function* () {
      const {
        query,
        foundSearchPackages,
        foundSuggestionsPackages,
        selectedPackages,
        focused
      } = _this.state;

      switch (key.name) {
        case 'tab':
          if (focused === FOCUSED_SEARCH && (0, _utils.notEmpty)(foundSuggestionsPackages)) {
            _this.setState({
              focused: FOCUSED_SUGGESTIONS
            });
          } else if (focused === FOCUSED_SUGGESTIONS && (0, _utils.notEmpty)(foundSearchPackages) && (0, _utils.notEmpty)(query)) {
            _this.setState({
              focused: FOCUSED_SEARCH
            });
          }

          break;

        case 'space':
          _this.handleInstall();

          break;

        case 'backspace':
          if ((0, _utils.isEmpty)(query)) {
            yield _this.setState({
              selectedPackages: selectedPackages.slice(0, -1)
            });

            _this.fetchSuggestions();
          }

        default:
          break;
      }
    })();
  }

  handleQueryChange(query) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      //check if tab or space have been pressed
      if (/\s/g.test(query)) {
        return;
      }

      if ((0, _utils.isEmpty)(query)) {
        _this2.setState({
          query,
          focused: FOCUSED_SUGGESTIONS
        });

        return;
      }

      yield _this2.setState({
        query
      });

      _this2.fetchSearch();
    })();
  }

  fetchSearch() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const {
        query
      } = _this3.state;
      const {
        limit
      } = _this3.props;

      _this3.setState({
        focused: FOCUSED_SEARCH,
        loadingSearchBackground: true
      });

      setTimeout(() => {
        if (_this3.state.loadingSearchBackground && _this3.state.query == query) {
          _this3.setState({
            loadingSearch: PROGRESS_LOADING,
            loadingSearchBackground: false
          });
        }
      }, 500);

      try {
        const hits = yield (0, _algoliaSearch.getSearch)(query, limit);
        const cells = (0, _utils.hitsToCells)(hits);

        if (_this3.state.query === query) {
          _this3.setState({
            foundSearchPackages: cells
          });
        }

        _this3.setState({
          loadingSearch: PROGRESS_LOADED,
          loadingSearchBackground: false
        });
      } catch (err) {
        _this3.setState({
          loadingSearch: PROGRESS_ERROR,
          loadingSearchBackground: false
        });
      }
    })();
  }

  handleTogglePackage(pkg) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      const {
        selectedPackages: selectedPackagesOld
      } = _this4.state;
      const exists = selectedPackagesOld.some(({
        objectID
      }) => objectID === pkg.objectID);

      if (exists) {
        return;
      }

      yield _this4.setState({
        query: '',
        selectedPackages: [...selectedPackagesOld, pkg]
      });

      _this4.fetchSuggestions();
    })();
  }

  fetchSuggestions() {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      const {
        selectedPackages
      } = _this5.state;
      const {
        dev: isDev,
        limit
      } = _this5.props;

      _this5.setState({
        focused: FOCUSED_SUGGESTIONS,
        loadingSuggestions: PROGRESS_LOADING
      });

      try {
        const hits = yield (0, _npmSuggestions.getSuggestions)(selectedPackages.map(selectedPackage => selectedPackage.name), limit, isDev);
        const cells = (0, _utils.hitsToCells)(hits);

        _this5.setState({
          foundSuggestionsPackages: cells,
          loadingSuggestions: PROGRESS_LOADED
        });
      } catch (err) {
        _this5.setState({
          loadingSuggestions: PROGRESS_ERROR
        });
      }
    })();
  }

  handleInstall() {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      const {
        selectedPackages
      } = _this6.state;

      if ((0, _utils.isEmpty)(selectedPackages)) {
        _this6.props.onExit();
      } // ENV


      const isDev = _this6.props.dev;
      const yarn = yield (0, _utils.shouldUseYarn)();
      const env = yarn ? 'yarnpkg' : 'npm';
      const arg = yarn ? ['add'] : ['install', '--save'];
      const devArg = yarn ? '-D' : '--save-dev'; // Packages

      const packages = selectedPackages.map(pkg => pkg.name);
      const args = [...arg, ...packages, ...(isDev ? [devArg] : [])]; // Install the queries

      try {
        yield _execa.default.sync(env, args, {
          stdio: `inherit`
        });
      } catch (err) {
        throw err;
      }

      _this6.props.onExit();
    })();
  }

}

var _default = Emma;
exports.default = _default;