#!/usr/bin/env node
"use strict";

var _meow = _interopRequireDefault(require("meow"));

var _ink = require("ink");

var _updateNotifier = _interopRequireDefault(require("update-notifier"));

var _emma = _interopRequireDefault(require("./emma"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Notify updater
const pkg = require(`../package.json`);

(0, _updateNotifier.default)({
  pkg
}).notify(); // CLI

const cli = (0, _meow.default)(`
  Usage
    $ emma

  Options
    --dev -D      Add to dev dependencies.
    --limit -L      Number of packages shown, defaults to 5.

  Example
    $ emma -D

  Run without package-name to enter live search.
  Use keyboard to search through package library.
  Use up/down to select packages.
  Use enter to select a package.
  Use tab to move between search/suggestions.
  When query is empty use backspace to remove packages.
  Click space to trigger the install.   
`, {
  flags: {
    dev: {
      type: 'boolean',
      alias: 'D'
    },
    limit: {
      type: 'string',
      alias: 'L',
      default: '5'
    }
  }
});

const main = () => {
  let unmount; // eslint-disable-line prefer-const

  const onError = () => {
    unmount();
    process.exit(1);
  };

  const onExit = () => {
    unmount();
    process.exit();
  };

  const {
    dev,
    limit
  } = cli.flags; // Uses `h` instead of JSX to avoid transpiling this file

  unmount = (0, _ink.render)((0, _ink.h)(_emma.default, {
    dev,
    limit,
    onError,
    onExit
  }));
};

main();