"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getSuggestions = getSuggestions;
exports.devSuggestionsQuery = exports.suggestionsQuery = void 0;

var _apolloClient = require("apollo-client");

var _apolloCacheInmemory = require("apollo-cache-inmemory");

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _apolloLinkHttp = require("apollo-link-http");

var _graphqlTag = _interopRequireDefault(require("graphql-tag"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const client = new _apolloClient.ApolloClient({
  link: new _apolloLinkHttp.HttpLink({
    uri: 'https://npm-suggestions.now.sh',
    fetch: _nodeFetch.default
  }),
  cache: new _apolloCacheInmemory.InMemoryCache()
});
const suggestionsQuery = _graphqlTag.default`
  query Suggestions(
    $dependencies: [String!]!
    $devDependencies: [String!]!
    $limit: Int
  ) {
    suggestions(
      dependencies: $dependencies
      devDependencies: $devDependencies
      limit: $limit
    ) {
      suggestions {
        name
        description
        version
        owner {
          name
        }
        humanDownloadsLast30Days
        objectID
      }
    }
  }
`;
exports.suggestionsQuery = suggestionsQuery;
const devSuggestionsQuery = _graphqlTag.default`
  query Suggestions(
    $dependencies: [String!]!
    $devDependencies: [String!]!
    $limit: Int
  ) {
    suggestions(
      dependencies: $dependencies
      devDependencies: $devDependencies
      limit: $limit
    ) {
      devSuggestions {
        name
        description
        version
        owner {
          name
        }
        humanDownloadsLast30Days
        objectID
      }
    }
  }
`;
exports.devSuggestionsQuery = devSuggestionsQuery;

function getSuggestions(_x, _x2, _x3) {
  return _getSuggestions.apply(this, arguments);
}

function _getSuggestions() {
  _getSuggestions = _asyncToGenerator(function* (dependencies, limit, dev) {
    const res = yield client.query({
      query: dev ? devSuggestionsQuery : suggestionsQuery,
      variables: {
        dependencies: dev ? [] : dependencies,
        devDependencies: dev ? dependencies : [],
        limit
      }
    });
    const hits = dev ? res.data.suggestions.devSuggestions : res.data.suggestions.suggestions;
    return hits;
  });
  return _getSuggestions.apply(this, arguments);
}