"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hitsToCells = exports.isEmpty = exports.notEmpty = exports.shouldUseYarn = exports.maxCellSize = void 0;

var _termSize = _interopRequireDefault(require("term-size"));

var _util = require("util");

var _fs = _interopRequireDefault(require("fs"));

var _child_process = _interopRequireDefault(require("child_process"));

var _dotProp = _interopRequireDefault(require("dot-prop"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

// Terminal
const maxCellSize = () => (0, _termSize.default)().columns / 4; // Yarn


exports.maxCellSize = maxCellSize;
const canAccessFile = (0, _util.promisify)(_fs.default.access);
const exec = (0, _util.promisify)(_child_process.default.exec);

const isYarnInstalled =
/*#__PURE__*/
function () {
  var _ref = _asyncToGenerator(function* () {
    try {
      yield exec(`yarnpkg --version`, {
        stdio: `ignore`
      });
      return true;
    } catch (err) {
      return false;
    }
  });

  return function isYarnInstalled() {
    return _ref.apply(this, arguments);
  };
}();

const shouldUseYarn =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(function* () {
    try {
      yield canAccessFile('package-lock.json');
      return false;
    } catch (err) {
      return isYarnInstalled();
    }
  });

  return function shouldUseYarn() {
    return _ref2.apply(this, arguments);
  };
}(); // Additional


exports.shouldUseYarn = shouldUseYarn;

const notEmpty = x => x.length !== 0;

exports.notEmpty = notEmpty;

const isEmpty = x => x.length === 0;

exports.isEmpty = isEmpty;

const getCellPadding = (hits, hit) => attr => {
  const cells = hits.map(_hit => _dotProp.default.get(_hit, attr));
  const cellWidth = Math.max(...cells.map(cell => cell ? cell.length : 0));
  const cellValueWidth = _dotProp.default.get(hit, attr) === null ? 0 : _dotProp.default.get(hit, attr).length;
  const width = cellWidth - cellValueWidth;
  return ` `.repeat(width);
};

const hitsToCells = hits => hits.map(hit => _objectSpread({}, hit, {
  _cell: getCellPadding(hits, hit)
}));

exports.hitsToCells = hitsToCells;